#region Version Information

// ///////////////////////////////////////////////////////////////////////////
// Module: Base.cs
// 
// Description:
// ------------
// Base implementations for a Syncfusion Diagram SymbolModel and Symbol to
// be associated with dynamic reporting object types.
// 
// Author: is
// Date created: 25.02.2004
// 
// Version Control Information:
// ----------------------------
// $Revision: 5 $
// $Author: Is $
// $Date: 5/10/04 10:03a $
// $Workfile: Base.cs $
// 
// Version History
// ---------------
// $History: Base.cs $
// 
// *****************  Version 5  *****************
// User: Is           Date: 5/10/04    Time: 10:03a
// Updated in $/Actuate/ReportObjectTypes/Base
// Moved all object types to the single DLL
// 
// *****************  Version 4  *****************
// User: Is           Date: 13.04.04   Time: 8:20
// Updated in $/Actuate/ReportObjectTypes/Base
// Removed Trace output
// 
// *****************  Version 3  *****************
// User: Manmic       Date: 4/03/04    Time: 9:26
// Updated in $/Actuate/ReportObjectTypes/Base
// Added Icons for each symbol
// 
// *****************  Version 2  *****************
// User: Is           Date: 1.03.04    Time: 14:29
// Updated in $/Actuate/ReportObjectTypes/Base
// Creatly improved base implementation and changed child controls
// accordingly
// 
// *****************  Version 1  *****************
// User: Is           Date: 25.02.04   Time: 17:31
// Created in $/Actuate/ReportObjectTypes/Base
//
// $NoKeyWords: $

#endregion


#region Usings

using System;
using System.Diagnostics;
using System.Runtime;
using System.Reflection;
using System.IO;
using System.Collections;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Drawing.Imaging;
using System.ComponentModel;

using Syncfusion.Windows.Forms.Diagram;
using Syncfusion.Windows.Forms.Diagram.Controls;
using Sync = Syncfusion.Windows.Forms.Diagram;

#endregion


namespace DWS.ITMS.Actuate.ObjectType
{
	#region Symbol Model Base

	/// <summary>
	/// Base implementation for a Syncfusion Diagram SymbolModel to
	/// be associated with dynamic reporting object types.
	/// </summary>
	public class SymbolModelBase : SymbolModel
	{
		#region Constants

		internal const string OBJECT_TYPE_HINT	= "ObjectTypeHint";
		internal const string OBJECT_HINT		= "ObjectHint";
		internal const string LABEL_HINT		= "LabelHint";

		#endregion


		#region Properties

		/// <summary>
		/// Gets or sets the symbol's label (shown on the symbol)
		/// </summary>
		public string Label
		{
			get { return(GetPropertyValue(LABEL_HINT) as string); }
			set { SetPropertyValue(LABEL_HINT, value); }
		}
		
		#endregion


		#region Constructor Destructor

		/// <summary>
		/// Standard Constructor.
		/// </summary>
		public SymbolModelBase(object objectType, Type symbolType, object obj)
		{
			try
			{
				LargeIcon		= LoadImage("IconLarge.bmp");
				SmallIcon		= LoadImage("IconSmall.bmp");

				// Got no large icon but a small one?
				if(LargeIcon == null && SmallIcon != null)
				{
					LargeIcon	= ResizeBitmap(SmallIcon, 32, 32);
				}

				// Got no small icon but a large one?
				else if(SmallIcon == null && LargeIcon != null)
				{
					SmallIcon	= ResizeBitmap(LargeIcon, 16, 16);
				}

				// Set the object type
				SetPropertyValue(OBJECT_TYPE_HINT, objectType);
				SetPropertyValue(OBJECT_HINT, obj);
				
				string	label	= symbolType.Name;
				if(label.EndsWith("Symbol") == true)
				{
					label	= label.Substring(0, label.Length - 6);
				}
				SetPropertyValue(LABEL_HINT, label);

				base.Name		= label;

				// Specify the Plugin Assembly and Class settings
				PlugInAssembly		= symbolType.Assembly.GetName().Name;
				PlugInClass			= symbolType.FullName;
			}
			catch(Exception x)
			{
				throw x;
			}
		}

		#endregion


		#region Methods

		/// <summary>
		/// Load in a resource image from the instance's assembly
		/// </summary>
		/// <param name="name">Name of the image to load.</param>
		/// <returns>Reference to the <see cref="Bitmap"/> or <strong>null</strong> on failure.</returns>
		private Bitmap LoadImage(string name)
		{
			Bitmap	image		= null;
			Type	execType	= GetType();
			int		index		= execType.Namespace.LastIndexOf('.');
			string	resource	= execType.Namespace.Substring(0, index) + ".Resources." + 
				execType.Namespace.Substring(index + 1) + "." + name;
			Stream	stream		= execType.Assembly.GetManifestResourceStream(resource);
			if(stream != null)
			{
				image	= (Bitmap)Bitmap.FromStream(stream);
			}
			return(image);
		}

		/// <summary>
		/// Resize a given bitmap to a new size.
		/// </summary>
		/// <param name="original">The original <see cref="Image"/> to resize.</param>
		/// <param name="width">New width of the bitmap.</param>
		/// <param name="height">New height of the bitmap.</param>
		/// <returns>The resized copy of the <see cref="Bitmap"/></returns>
		private Bitmap ResizeBitmap(System.Drawing.Image original, int width, int height)
		{
			return(new Bitmap(original, new Size(width, height)));
		}

		#endregion
	}

	#endregion



	#region Symbol Base

	/// <summary>
	/// Base implementation for a Syncfusion Diagram Symbol to
	/// be associated with dynamic reporting object types.
	/// </summary>
	public abstract class SymbolBase : Symbol
	{
		#region Constants

		protected const int INITIAL_WIDTH = 180;
		protected const int INITIAL_HEIGHT = 120;

		#endregion


		#region Variables

		protected Hashtable				_properties		= null;
		protected System.Drawing.Image	_texture		= null;

		#endregion


		#region Properties

		/// <summary>
		/// Gets or sets the reference to the object type hint
		/// </summary>
		[Browsable(false)]
		public object ObjectTypeHint
		{
			get { return(GetPropertyValue(SymbolModelBase.OBJECT_TYPE_HINT)); }
			set { SetPropertyValue(SymbolModelBase.OBJECT_TYPE_HINT, value); }
		}

		/// <summary>
		/// Gets or sets the reference to the object hint
		/// </summary>
		[Browsable(false)]
		public object ObjectHint
		{
			get { return(GetPropertyValue(SymbolModelBase.OBJECT_HINT)); }
			set { SetPropertyValue(SymbolModelBase.OBJECT_HINT, value); }
		}

		/// <summary>
		/// Gets or sets the symbol's label (shown on the symbol)
		/// </summary>
		[Browsable(false)]
		public string Label
		{
			get { return(GetPropertyValue(SymbolModelBase.LABEL_HINT) as string); }
			set { SetPropertyValue(SymbolModelBase.LABEL_HINT, value); }
		}

		/// <summary>
		/// Access the image used as a texture at runtime for the symbol representation
		/// </summary>
		public System.Drawing.Image Texture
		{
			get { return _texture;}
			set { _texture = value;}
		}

		#endregion


		#region Constructor Destructor

		/// <summary>
		/// Standard Constructor
		/// </summary>
		public SymbolBase() : base()
		{
			_properties			= new Hashtable(3);
			_properties.Add(SymbolModelBase.LABEL_HINT, null);
			_properties.Add(SymbolModelBase.OBJECT_TYPE_HINT, null);

			// Preset the symbol property for the object type hint
			ObjectTypeHint		= null;
			ObjectHint			= null;
			Label				= null;
		}

		#endregion


		#region Methods

		/// <summary>
		/// Initialize the symbol instance given some hint object.
		/// </summary>
		public virtual void InitSymbol()
		{
			// Clear all children
			base.children.Clear();

			// Draw background
			Syncfusion.Windows.Forms.Diagram.Rectangle	rect;

			rect	= new Syncfusion.Windows.Forms.Diagram.Rectangle(0, 0, INITIAL_WIDTH, INITIAL_HEIGHT);
			rect.Name					= "SymbolBack";
			rect.LineStyle.LineColor	= Color.Transparent;
			rect.FillStyle.Color		= Color.FromArgb(255, 240, 240, 240);
			rect.FillStyle.Type			= FillStyle.FillType.Solid;
			AppendChild(rect);

			// Draw the label
			AddLabel(Label);
		}

		/// <summary>
		/// One of the symbol's propertie's was changed.
		/// </summary>
		/// <param name="evtArgs"></param>
		protected override void OnPropertyChanged(PropertyEventArgs evtArgs)
		{
			base.OnPropertyChanged(evtArgs);

			bool complete = true;

			// is it a needed property
			if (_properties.Contains(evtArgs.PropertyName) == true && evtArgs.NewValue != null)
			{
				_properties[evtArgs.PropertyName] = evtArgs.NewValue;
			}

			// test for completeness
			foreach(object key in _properties.Keys)
			{
				complete &= (_properties[key] != null);
			}
			
			// initialize if complete
			if (complete == true)
			{
				InitSymbol();
			}
		}

		/// <summary>
		/// Add a label to the symbol.
		/// </summary>
		/// <remarks>
		/// The label will be aligned centered below the symbol
		/// </remarks>
		/// <param name="text">The symbol's label</param>
		public void AddLabel(string text)
		{
			SymbolLabel	label			= new SymbolLabel();
			label.Text					= text;
			label.FontStyle.Family		= "Tahoma";
			label.FontStyle.Size		= 10;
			label.FontStyle.Bold		= true;
			label.ReadOnly				= true;
			label.Anchor				= BoxPosition.TopLeft;
			label.OffsetX				= 0;
			label.OffsetY				= -10;
			label.Location				= new PointF(0, 0);
			label.HorizontalAlignment	= StringAlignment.Near;
			label.VerticalAlignment		= StringAlignment.Near;
			label.EditStyle.AllowMove	= false;
			label.EditStyle.AllowSelect	= false;
			label.EditStyle.AllowResize	= true;
			label.EditStyle.Enabled		= false;

			base.AddLabel(label);

			label.OffsetX				= label.Width / 2;
		}

		/// <summary>
		/// Add the texture to the texturePanel
		/// </summary>
		/// <param name="texture">the texture to use in the panel</param>
		/// <param name="origin">the starting poins of the texturePanel</param>
		protected void AddTexture(System.Drawing.Image texture, PointF origin)
		{
			Sync.Rectangle back			= children["SymbolBack"] as Sync.Rectangle;
			Sync.Rectangle iconPanel	= children["IconPanel"] as Sync.Rectangle;

			if (iconPanel != null)
			{
				children.Remove(iconPanel);
			}
			
			Sync.Rectangle newPanel = new Sync.Rectangle(origin.X, origin.Y, 49, 49);
            
			newPanel.Name						= "IconPanel";
			newPanel.LineStyle.LineColor		= Color.White;
			newPanel.FillStyle.TextureWrapMode	= System.Drawing.Drawing2D.WrapMode.Tile;
			newPanel.FillStyle.Texture			= texture;		
			
			this.AppendChild(newPanel);
		}


		#endregion
	}

	#endregion
}
